#include <iostream>
#include "binaryTree.hh"

// Check if two subtrees are symmetric
bool isSymmetric(Node* aNode1, Node* aNode2) {
  if (!aNode1 && !aNode2) return true;
  if (!aNode1  || !aNode2 ) return false;
  return (aNode1->getLabel() == aNode2->getLabel())
    && isSymmetric(aNode1->getRightChild(), aNode2->getLeftChild())
    && isSymmetric(aNode1->getLeftChild(), aNode2->getRightChild());
}

// check if a tree is symmetric
bool isSymmetric(Node* aRootNode) {
  return isSymmetric(aRootNode, aRootNode);
}

// invert the given tree
Node* invertTree(Node* aNode) {
    if (!aNode) {
      return nullptr;
    }

    Node* lRight = invertTree(aNode->getRightChild());
    Node* lLeft = invertTree(aNode->getLeftChild());
    aNode->setLeftChild(lRight);
    aNode->setRightChild(lLeft);
    return aNode;
}

// flatten to linked list
void flatten(Node* aNode) {
  if (!aNode) { return; }           // Recursion anchor: nullptr nodes

  flatten(aNode->getRightChild());  // Recursively flatten both child nodes
  flatten(aNode->getLeftChild());   // => _left and _right are now chains!

  /* 
   * Go to the rightmost (deepest) node in the left subtree of aNode 
   * and append the entire right subtree of aNode as rightmost's right child. 
   */
  Node* lRightMost = aNode->getLeftChild();
  if (lRightMost) {
    while (lRightMost->getRightChild()) {
      lRightMost = lRightMost->getRightChild();
    }

    lRightMost->setRightChild(aNode->getRightChild());  
    aNode->setRightChild(aNode->getLeftChild());
    aNode->setLeftChild(nullptr);
  }
}

void foo(Node aNode) {
  std::cout << "FOO" << std::endl;
  (void) aNode;  // Silence compiler warning
}

void bar(const Node& aNode) {
  std::cout << "BAR" << std::endl;
  (void) aNode;  // Silence compiler warning
}


int main() {
  Node lRoot(0);
  Node* lLeft = new Node(1);
  Node* lRight = new Node(2);;
  Node* lLeftLeft = new Node(3);
  Node* lLeftRight = new Node(4);;
  Node* lRightLeft = new Node(5);
  Node* lRightRight = new Node(6);

  lRoot.setLeftChild(lLeft);
  lRoot.setRightChild(lRight);
  lLeft->setLeftChild(lLeftLeft);
  lLeft->setRightChild(lLeftRight);
  lRight->setLeftChild(lRightLeft);
  lRight->setRightChild(lRightRight);

  lRoot.print();
  std::cout << std::endl;

  foo(lRoot);
  bar(lRoot);

  std::cout << isSymmetric(&lRoot) << std::endl;
  Node* lInvertedTree = invertTree(&lRoot);

  lInvertedTree->print();
  std::cout << std::endl;

  flatten(lInvertedTree);
  lInvertedTree->print();
  std::cout << std::endl;
}


