/*
 * A simple compression function for strings
 * Example: compress aaabbcdd to a3b2cd2
 * If the compressed string is larger than the original,
 * the function may return the original string.
 */

#include <iostream>
#include <string>

/* Compression function */
std::string compress (const std::string& s) {
  unsigned beg     = 0;
  unsigned end     = 0;
  unsigned counter = 0;
  std::string res  = "";  // The resulting compression

  while (end < s.size()) {
    if (s[beg] != s[end]) {
      if (counter > 1) {
        res += s[beg] + std::to_string(counter);  // more than one occurrence of the character
      } else {
        res += s[beg];                            // only one occurrence of the character
      }
      counter = 0;
      beg = end;
    }
    ++counter;
    ++end;
  }

  if (counter > 1) {
    res += s[beg] + std::to_string(counter);
  } else {
    res += s[beg];
  }

  return res;
}


std::string compressPtr (const std::string& s) {
  const char* beg = &s[0];
  const char* end = &s[0];
  unsigned counter = 0;

  std::string res = "";

  while (static_cast<size_t>(end - &s[0]) < s.size()) {
    if (*beg != *end) {
      if (counter > 1) {
        res += *beg + std::to_string(counter);
      } else {
        res += *beg;
      }
      counter = 0;
      beg = end;
    }
    ++counter;
    ++end;
  }

  if (counter > 1) {
    res += *beg + std::to_string(counter);
  } else {
    res += *beg;
  }

  return res;
}

int main (int argc, char* argv []) {
  if (argc != 2) {
    std::cout << "usage: " << argv[0] << " <string>";
    return 111;
  }
  std::string s (argv[1]);
  std::cout << "input string: " << s
            << "\ncompressed string:       " << compress(s)
            << "\ncompressed string (ptr): " << compressPtr(s)
            << std::endl;
}
